import { __awaiter } from "tslib";
import { chain, SchematicsException, } from '@angular-devkit/schematics';
import { addImportToModule, insertImport, } from '@schematics/angular/utility/ast-utils';
import * as ts from 'typescript';
import { packageName, findRootModule, getProject, } from '../../common';
function getModuleFile(tree, options) {
    const modulePath = options.module;
    if (!tree.exists(modulePath)) {
        throw new SchematicsException(`File ${modulePath} does not exist.`);
    }
    const text = tree.read(modulePath);
    if (text === null) {
        throw new SchematicsException(`File ${modulePath} does not exist.`);
    }
    const sourceText = text.toString('utf-8');
    return ts.createSourceFile(modulePath, sourceText, ts.ScriptTarget.Latest, true);
}
function applyChanges(tree, path, changes) {
    const recorder = tree.beginUpdate(path);
    for (const change of changes) {
        recorder.insertLeft(change.pos, change.toAdd);
    }
    tree.commitUpdate(recorder);
    return tree;
}
function addImportsToModuleFile(options, imports, importName = packageName) {
    return (tree) => {
        const module = getModuleFile(tree, options);
        const importChanges = insertImport(module, options.module, imports.join(', '), importName);
        return applyChanges(tree, options.module, [
            importChanges,
        ]);
    };
}
function addImportsToModuleDeclaration(options, imports) {
    return (tree) => {
        const module = getModuleFile(tree, options);
        const importChanges = imports.map((imp) => addImportToModule(module, options.module, imp, packageName)[0]);
        return applyChanges(tree, options.module, importChanges);
    };
}
export function ngAdd(options) {
    return (tree) => __awaiter(this, void 0, void 0, function* () {
        var _a;
        const project = yield getProject(tree, options.project);
        const sourceRoot = (_a = (project && project.sourceRoot)) !== null && _a !== void 0 ? _a : 'src';
        const modulesToAdd = ['LetModule', 'PushModule'];
        options.module = findRootModule(tree, options.module, sourceRoot);
        return chain([
            addImportsToModuleFile(options, modulesToAdd),
            addImportsToModuleDeclaration(options, modulesToAdd),
        ]);
    });
}
//# sourceMappingURL=index.js.map